%this code should work with any linearized model that has a monetary policy
%rule in it. It should be called right after the stoch_simul command. 

%the code will simulate the effects of keeping the nominal interest rate
%constant for a while, to be specified by the user. This forecast is labeled 
%the 'Odyssean Fixed Interest Rate Simulation'  The agents
%know at time T that the interest rates will be held fixed for periods
%T+1,...,T+h

%It will also construct a smoothed conditional expectation where the interest
%rate is fixed at its level at time T for the subsequent h periods.  The agents at time 
%T did not know the interest rate would be fixed for these next h periods.

%Prepared by Patrick Higgins, Atlanta Fed, 8/20/2017

%<<>>Load Command not needed, these are the parameters chosen in RunFixedInterestRateNewB.m
%load ForeDataParams FixInterestRateVect forelength PolicyRateName MonetaryShockName EqnR;
NumberOfPeriodsWithFixIR = length(FixInterestRateVect);

eval(['insampsize = length(' deblank(char(M_.endo_names(1,:))) ') - forelength;']);



% eliminate shocks with 0 variance
i_exo_var = setdiff([1:M_.exo_nbr],find(diag(M_.Sigma_e) == 0));
nxs = length(i_exo_var);
chol_S = chol(M_.Sigma_e(i_exo_var,i_exo_var));


indINTRATE = strmatch(PolicyRateName,M_.endo_names(oo_.dr.order_var,:),'exact');
indMONPOLSHOCK = strmatch(MonetaryShockName,M_.exo_names,'exact');

xlen = M_.maximum_endo_lead + M_.maximum_endo_lag + 1;

klen = M_.maximum_lag + M_.maximum_lead + 1;
iyv = M_.lead_lag_incidence';
iyv = iyv(:);
iyr0 = find(iyv) ;
it_ = M_.maximum_lag + 1 ;
%Taken from line 155 of dr1
z = repmat(oo_.dr.ys,1,klen);

[junk,jacobia_] = feval([M_.fname '_dynamic'],z,[oo_.exo_simul ...
    oo_.exo_det_simul], M_.params, oo_.dr.ys, it_);

%Construct alpha0, alpha1, alpha2, Beta1.
%For now assume that s(t) is a vector of AR(1) shock processes
%and that the shock process equations appear last

%First we pad it with columns of zeros

for iii = 1:3
    fff=find(M_.lead_lag_incidence(iii,:)>0);
    ddd=diff(M_.lead_lag_incidence(iii,fff));
    if min(ddd) < 1 | max(ddd) > 1
        error('Fatal misunderstanding of M_.lead_lag_incidence');
    end
end

M_.lead_lag_incidenceT = M_.lead_lag_incidence';
M_.lead_lag_incidenceT(:);

if size(M_.endo_names,1) ~= size(jacobia_,1)
    error('Fatal misunderstanding of dimension of  jacobia_');
end

jacobiaDynareENDOpad = zeros(size(M_.endo_names,1),3*size(M_.endo_names,1) + size(M_.exo_names,1));
jacobiaDynareENDOpad(:,find(M_.lead_lag_incidenceT(:)>0))=...
    jacobia_(:,1:length(find(M_.lead_lag_incidenceT(:)>0)));

jacobiaDynareENDOpad(:,end-size(M_.exo_names,1)+1:end) = jacobia_(:,end-size(M_.exo_names,1)+1:end);

%Set up alpha0, alpha1, alpha2, Beta1.  Re order the elements so they are
%in the same order as M_.endo_names(oo_.dr.order_var,:)

for iii = 1:3
    AlphaStruct(3-iii+1).alpha = NaN(size(M_.endo_names,1));
    for rrr = 1:size(M_.endo_names,1)
        for mmm = 1:size(M_.endo_names,1)
            ccc = size(M_.endo_names,1)*(iii-1) + mmm;
            cccnew = oo_.dr.inv_order_var(mmm);
            AlphaStruct(3-iii+1).alpha(rrr,cccnew) = jacobiaDynareENDOpad(rrr,ccc);
        end
        
    end
end

alpha0 = AlphaStruct(1).alpha;
alpha1 = AlphaStruct(2).alpha;
alpha2 = AlphaStruct(3).alpha;

alpha0hat = alpha0;
alpha1hat = alpha1;
alpha2hat = alpha2;

alpha0hat(EqnR,:) = 0;
alpha2hat(EqnR,:) = 0;

tauChrist = zeros(1,size(alpha1hat,1));
tauChrist(indINTRATE) = 1;
alpha1hat(EqnR,:) = tauChrist;

beta1 = jacobiaDynareENDOpad(:,end-size(M_.exo_names,1)+1:end);
beta1hat = beta1;
beta1hat(EqnR,:) = 0;

%Set up A & B using oo_.dr.ghx and oo_.dr.ghu
%This is complicated by the fact that the Christiano and Dynare
%definitions of endogenous/exogenous variables are not the same


%nstatic + 1 and end at index nstatic + npred + 1

%A is
AChrist = zeros(size(M_.endo_names,1));
AChrist(:,M_.nstatic+1:M_.nstatic+M_.npred) = oo_.dr.ghx; %<<>>Patrick note -- 06-19-17.  Parameters oo_.dr.nstatic and oo_.dr.npred in 4.2.4 have been placed in M_.nstatic and M_.npred in 4.5.0

BChrist = oo_.dr.ghu;

d = zeros(size(M_.endo_names,1),NumberOfPeriodsWithFixIR);

eval(['d(EqnR,:) = ' PolicyRateName '(insampsize);']);
d(EqnR,:) = d(EqnR,:) + FixInterestRateVect;


AStruct(1).A = AChrist;
BStruct(1).B = BChrist;
dStruct(1).d = zeros(size(M_.endo_names,1),1);

for iii = 1:NumberOfPeriodsWithFixIR
    if rank((alpha0hat*AStruct(iii).A + alpha1hat)) < size(M_.endo_names,1)
       error('Matrix is not full rank');   
    end
    AStruct(iii+1).A = -inv(alpha0hat*AStruct(iii).A + alpha1hat)*alpha2hat;
    BStruct(iii+1).B = -inv(alpha0hat*AStruct(iii).A + alpha1hat);
    dStruct(iii+1).d = inv(alpha0hat*AStruct(iii).A + alpha1hat)*(d(:,iii) - alpha0hat*dStruct(iii).d);    
end

%Now simulate the two systems

data = [];

for iii = 1:size(M_.endo_names,1)
    eval(['data = [data ' M_.endo_names(iii,:) '];']);
end

%Reorder and drop observations before fixed interest rate period
data = data(1:end-forelength,oo_.dr.order_var);
dataactfore = [data ; NaN(forelength,length(oo_.dr.order_var))];

curstate = data(end,:)';

UnConditionalForecast = NaN(size(data,1)+forelength,size(data,2));
ConditionalForecast = NaN(size(data,1)+forelength,size(data,2));


UnConditionalForecast(size(data,1),:) = curstate';
ConditionalForecast(size(data,1),:) = curstate';

for iii = 1:NumberOfPeriodsWithFixIR
    UnConditionalForecast(size(data,1)+iii,:) = AChrist*(UnConditionalForecast(size(data,1)+iii-1,:)');
    ConditionalForecast(size(data,1)+iii,:) = AStruct(NumberOfPeriodsWithFixIR+2-iii).A*(ConditionalForecast(size(data,1)+iii-1,:)') + dStruct(NumberOfPeriodsWithFixIR+2-iii).d;
end


for iii = NumberOfPeriodsWithFixIR+1:forelength
    UnConditionalForecast(size(data,1)+iii,:) = AChrist*(UnConditionalForecast(size(data,1)+iii-1,:)');
    ConditionalForecast(size(data,1)+iii,:) = AChrist*(ConditionalForecast(size(data,1)+iii-1,:)');
end

